import os
from tempfile import mkdtemp

from AccessControl import ClassSecurityInfo

from Products.CMFCore import permissions
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.public import *
from Products.Archetypes.utils import contentDispositionHeader

from Products.ATContentTypes.content.image import ATImage, ATImageSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget

from zope.component import getAdapter
from latexmath2png import math2png

schema = \
  ATImageSchema.copy() + \
  Schema((
    StringField(
        'LatexMathExpression',
        #default = r'y^2=x^2+\alpha^2',   # how to manage '\' from UI?
        required = True,
        widget = StringWidget(
                    label='Latex Math Expression',
                    label_msgid = "label_latexmathexpr",
                    description_msgid="help_latexmathexpr",
                    description='',
                    i18n_domain = "latexmath",
                    size=60,
                    )
        ),
    IntegerField(
        'ImageScale',
        #default = r'y^2=x^2+\alpha^2',   # how to manage '\' from UI?
        required = False,
        default=1,
        widget = IntegerWidget(
                    label='Image Scale',
                    label_msgid = "label_image_scale",
                    description_msgid="help_image_scale",
                    description='Scale the output by a factor of SCALE. '
                                'Default: 1 = 100%',
                    i18n_domain = "latexmath",
                    size=2,
                    )
        ),
))


schema['image'].required = False
schema['image'].widget.visible = {'edit' : 'invisible', 'view' : 'invisible' }
schema['description'].schemata = 'default'
schema['allowDiscussion'].widget.visible = {'edit' : 'invisible', 'view' : 'invisible' }
schema['relatedItems'].widget.visible = {'edit' : 'invisible', 'view' : 'invisible' }

finalizeATCTSchema(schema)

class LatexMathImage(ATImage):
    """ An image rendering a latex math expression """

    schema = schema

    archetype_name             = 'Latex Math Image'
    portal_type                = 'Latex Math Image'
    meta_type                  = 'LatexMathImage'
    suppl_views                = ()
    global_allow               = 1
    allow_discussion           = 0
    typeDescription            = "An image rendering a latex math expression"
    typeDescMsgId              = 'description_edit_latex_math_image'
    default_view               = 'image_view'

    security       = ClassSecurityInfo()

    content_icon = "image_icon.gif"

    def _generateAndSetImage(self, input, handle):
        size =  int(self.REQUEST.form.get('ImageScale', self.getImageScale()))
        math2png(input, handle, [], '', size)
        f = open(os.path.join(handle,'1.png'))
        self.setImage(f.read())

    def setLatexMathExpression(self, value):
        """Set and render the expression"""
        self.getField('LatexMathExpression').set(self, value)
        input = [self.getLatexMathExpression(), ]
        handle = mkdtemp()
        try:
            self._generateAndSetImage(input, handle)
        except Exception, e:
            if str(e) == 'latex error':
                input = [r'\textrm{invalid formula}', ]
                self._generateAndSetImage(input, handle)
            else:
                raise e

registerType(LatexMathImage)
